@@ -28,9 +28,8 @@ hb_subset_fuzzer = os.path.join (top_builddir, "hb-subset-fuzzer" + EXEEXT)
 
 if not os.path.exists (hb_subset_fuzzer):
         if len (sys.argv) < 2 or not os.path.exists (sys.argv[1]):
-                print ("""Failed to find hb-subset-fuzzer binary automatically,
+                sys.exit ("""Failed to find hb-subset-fuzzer binary automatically,
 please provide it as the first argument to the tool""")
-                sys.exit (1)
 
         hb_subset_fuzzer = sys.argv[1]
 
@@ -42,8 +41,7 @@ valgrind = None
 if os.environ.get('RUN_VALGRIND', ''):
 	valgrind = shutil.which ('valgrind')
 	if valgrind is None:
-		print ("""Valgrind requested but not found.""")
-		sys.exit (1)
+		sys.exit ("""Valgrind requested but not found.""")
 	if libtool is None:
 		print ("""Valgrind support is currently autotools only and needs libtool but not found.""")
 
@@ -75,5 +73,4 @@ run_dir (os.path.join (srcdir, "..", "subset", "data", "fonts"))
 run_dir (os.path.join (srcdir, "fonts"))
 
 if fails:
-        print ("%i subset fuzzer related tests failed." % fails)
-        sys.exit (1)
+	sys.exit ("%d subset fuzzer related tests failed." % fails)
